window.Gleam.initSearch([{"doc":"glimt","title":"glimt","content":"# glimt\n\n[![Package Version](https://img.shields.io/hexpm/v/glimt)](https://hex.pm/packages/glimt)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/glimt/)\n\nA Gleam library for logging\n\n\n*Get a glimpse (glimt in Swedish) of what is going on inside your software*\n\n## Installation\n\n```sh\ngleam add glimt\n```\n\nDocumentation can be found at &lt;https://hexdocs.pm/glimt&gt;.\n\n## Quick start\n\nGlimt is a logging library for Gleam that aims to modular in terms of how log-messages are dispatched and serialize while providing some default implementations that can be used out-of-the-box.\n\nA simple way to start logging is to use the built-in stdout logger.\n\n```gleam\nlet example_logger = new_stdout(&quot;example_logger&quot;)\nexample_logger\n|&gt; info(&quot;Entering hyperspace&quot;)\n```\n\nThis will produce a human readable log-line in your terminal that looks something like this:\n\n&gt; &lt;span style=&quot;color:gray&quot;&gt;2022-12-25T15:56:40.244Z&lt;/span&gt; | &lt;span style=&quot;color:green&quot;&gt;INFO&lt;/span&gt; | &lt;span style=&quot;color:magenta&quot;&gt;example_logger(&lt;0.9.0&gt;)&lt;/span&gt; | Entering hyperspace\n\nOf course you can log with different levels. The default stdout logger will print all levels as seen in the following example.\n\n```gleam\nlet levels_logger = new_stdout(&quot;levels_logger&quot;)\nlevels_logger\n|&gt; trace(&quot;Hyperdrive subsystem A entering state PRE_ACTIVE&quot;)\nlevels_logger\n|&gt; debug(&quot;Hyperdrive subsystems states: A=PRE_ACTIVE, B=RUNNING&quot;)\nlevels_logger\n|&gt; info(&quot;Entering hyperspace&quot;)\nlevels_logger\n|&gt; warning(&quot;Hyperdrive subsystem A got unexpected command: Nil&quot;)\nlevels_logger\n|&gt; error(\n&quot;Exiting hyperspace due to system failure&quot;,\nError(&quot;Subsystem A quit unexpectedly&quot;),\n)\nlevels_logger\n|&gt; fatal(\n&quot;Could not reboot shields after exiting hyperspace&quot;,\nError([\n    &quot;ENEMY SHIP DETECTED&quot;, &quot;ACTIVATE_SHIELDS&quot;, &quot;REBOOT_SHIELDS&quot;,\n    &quot;REBOOT_FAILURE&quot;,\n]),\n)\n```\n\n&gt; &lt;span style=&quot;color:gray&quot;&gt;2022-12-25T16:18:51.542Z&lt;/span&gt; | &lt;span style=&quot;color:cyan;font-weight:lighter&quot;&gt;TRACE&lt;/span&gt; | &lt;span style=&quot;color:magenta&quot;&gt;levels_logger(&lt;0.9.0&gt;)&lt;/span&gt; | Hyperdrive subsystem A entering state PRE_ACTIVE\n&gt;\n&gt; &lt;span style=&quot;color:gray&quot;&gt;2022-12-25T16:18:51.542Z&lt;/span&gt; | &lt;span style=&quot;color:cyan&quot;&gt;DEBUG&lt;/span&gt; | &lt;span style=&quot;color:magenta&quot;&gt;levels_logger(&lt;0.9.0&gt;)&lt;/span&gt; | Hyperdrive subsystems states: A=PRE_ACTIVE, B=RUNNING\n&gt;\n&gt; &lt;span style=&quot;color:gray&quot;&gt;2022-12-25T16:18:51.542Z&lt;/span&gt; | &lt;span style=&quot;color:green&quot;&gt;INFO&lt;/span&gt; | &lt;span style=&quot;color:magenta&quot;&gt;levels_logger(&lt;0.9.0&gt;)&lt;/span&gt; | Entering hyperspace\n&gt;\n&gt; &lt;span style=&quot;color:gray&quot;&gt;2022-12-25T16:18:51.542Z&lt;/span&gt; | &lt;span style=&quot;color:yellow&quot;&gt;WARNING&lt;/span&gt; | &lt;span style=&quot;color:magenta&quot;&gt;levels_logger(&lt;0.9.0&gt;)&lt;/span&gt; | Hyperdrive subsystem A got unexpected command: Nil\n&gt;\n&gt; &lt;span style=&quot;color:gray&quot;&gt;2022-12-25T16:18:51.542Z&lt;/span&gt; | &lt;span style=&quot;color:red&quot;&gt;ERROR&lt;/span&gt; | &lt;span style=&quot;color:magenta&quot;&gt;levels_logger(&lt;0.9.0&gt;)&lt;/span&gt; | Exiting hyperspace due to system failure | Error(&quot;Subsystem A quit unexpectedly&quot;)\n&gt;\n&gt; &lt;span style=&quot;color:gray&quot;&gt;2022-12-25T16:18:51.560Z&lt;/span&gt; | &lt;span style=&quot;color:red;font-weight:bold&quot;&gt;FATAL&lt;/span&gt; | &lt;span style=&quot;color:magenta&quot;&gt;levels_logger(&lt;0.9.0&gt;)&lt;/span&gt; | Could not reboot shields after exiting hyperspace | Error([&quot;ENEMY SHIP DETECTED&quot;, &quot;ACTIVATE_SHIELDS&quot;, &quot;REBOOT_SHIELDS&quot;, &quot;REBOOT_FAILURE&quot;])\n\nThe example above also shows how to attach error information to a log-message. Errors can be attached to error and fatal logs and can be any Error result. The default serializer will use `string.inspect` to print the error to the terminal.\n\n# Usage\n\n## Building a logger\n\nThe base type in Glimt is the `Logger`. A logger can have a name, a min level (messages below this level will not be logged at all). Futhermore the logger can be associated with one or more `instances`. An instance in responsible for dispatching log-messages to a specific target and in a specific format. The examples above used the `stdout-dispatcher` combined with the `basic-serializer` and the `new_stdout` is a convenience method for getting a logger with that specific setup. However it is also possible to start with an empty logger and the customize it in order to get more granular control. The following example builds the exact same logger as in the first example.\n\n```gleam\nimport gleam/option.{None}\nimport glimt.{ALL, TRACE, Direct}\nimport glimt/log_message.{level_value}\nimport glimt/serializer/basic.{basic_serializer}\nimport glimt/dispatcher/stdout.{dispatcher}\n\nlet custom_logger =\nnew(&quot;custom_logger&quot;)\n|&gt; level(ALL)\n|&gt; append_instance(Direct(\n    None,\n    level_value(TRACE),\n    dispatcher(basic_serializer),\n))\ncustom_logger\n|&gt; info(&quot;Entering hyperspace&quot;)\n```\n\n## Dispatchers and serializers\n\nThis example shows some important aspects of the Glimt architecture. First of all we can see the the stdout-dispatcher that was mentioned is created by providing a serializer (in this case the `basic_serializer`). However the dispatcher is just a function that takes a LogMessage and returns Nil (`fn (LogMessage) -&gt; Nil`). Futhermore the built-in `stdout-dispatcher` in created using a serializer which is a function that turns a `LogMessage` into a `String`. This makes is very easy to plug in specialized serializers into a logger. For example one could build a serializer that only prints the message field of the log_message.\n\n```gleam\nlet message_logger =\nnew(&quot;message_logger&quot;)\n|&gt; level(ALL)\n|&gt; append_instance(Direct(\n    None,\n    level_value(TRACE),\n    dispatcher(fn(log_message) { log_message.message }),\n))\nmessage_logger\n|&gt; warning(\n&quot;An important message to all our travelers. We are being attacked by enemy ships.&quot;,\n)\n```\n\n## Dispatching strategies\n\nAnother important aspect of the above examples is that the instance is constructed with the `Direct` constructor. This is one of the two dispatching strategies that Glimt supports and it means that the dispatcher will run synchronously in the same process as the code that logs the message. The other strategy is enabled by the `Actor` logger instance constructor and enables the actual dispatching of log-messages to be executed in a different process and only triggered by a message sent from the current process. The following example shows how we can utilize the convenience method `start_instance` to start an instance that runs in a separate process and logs to stdout.\n\n```gleam\nassert Ok(actor_instance) =\nstart_instance(&quot;actor_instance&quot;, TRACE, dispatcher(basic_serializer))\n\nlet actor_logger =\nnew(&quot;actor_logger&quot;)\n|&gt; level(INFO)\n|&gt; append_instance(actor_instance)\n\nactor_logger\n|&gt; trace(&quot;Cutting the blue wire&quot;)\nactor_logger\n|&gt; info(&quot;Bomb disarmed 💣&quot;)\n```\n\n&gt; &lt;span style=&quot;color:gray&quot;&gt;2022-12-25T18:11:41.608Z&lt;/span&gt; | &lt;span style=&quot;color:green&quot;&gt;INFO&lt;/span&gt; | &lt;span style=&quot;color:magenta&quot;&gt;actor_logger(&lt;0.9.0&gt;)/actor_instance(&lt;0.87.0&gt;)&lt;/span&gt; | Bomb disarmed 💣\n\nNote that both the calling pid and the actor pid are logged together with the name of the logger and logger instance. Another interesting aspect of this example is that the accepted log-level is set both for the logger and the instance. The level of the logger will be checked first and this is why we only see one of the messages logged in the example.\n\n## JSON\nGlimt comes with a build in JSON serializer (using [gleam_json](https://github.com/gleam-lang/json)). It can be used together with any dispatcher to dispatch logs in JSON format. The following example show how to output JSON to stdout.\n\n```gleam\nlet json_logger =\nnew(&quot;json_logger&quot;)\n|&gt; level(TRACE)\n|&gt; append_instance(Direct(\n    None,\n    level_value(TRACE),\n    dispatcher(new_json_serializer()),\n))\njson_logger\n|&gt; info(&quot;Hi Hypsipyle, it&#39;s me Thoas&quot;)\n```\n\n```json\n{&quot;time&quot;:&quot;2022-12-25T20:11:20.350Z&quot;,&quot;name&quot;:&quot;json_logger&quot;,&quot;pid&quot;:&quot;&lt;0.9.0&gt;&quot;,&quot;instance_name&quot;:null,&quot;instance_pid&quot;:null,&quot;level&quot;:&quot;INFO&quot;,&quot;message&quot;:&quot;Hi Hypsipyle, it&#39;s me Thoas&quot;,&quot;error&quot;:null}\n```\n\n## Additional data\n\nIf there is a need to attach additional data to the log-message it is possible to do this with the `log_with_data` function. This means that all logger instances of the logger need to accept data of a specific type. One easy way to accomplish this is to use the `json_serializer_with_data` which accepts an additional serializer function which should take a parameter of the specified type and return a `gleam_json` Json-type which will be used to serialize the data and insert under the `&quot;data&quot;`-key in the JSON log-message. The following example show how to do this.\n\nLet say we have a type like this:\n\n```gleam\ntype Data {\n  Data(user_id: String, trace_id: String)\n}\n```\n\nIn order to serialize the additional data we can use the builder in the `json` module to attach a serializer for the custom data-type. To log the data the `log_with_data` should be used.\n\n```gleam\nlet data_serializer = fn(data: Data) {\ngjson.object([\n    #(&quot;user_id&quot;, gjson.string(data.user_id)),\n    #(&quot;trace_id&quot;, gjson.string(data.trace_id)),\n])\n}\n\nlet data_logger =\nnew(&quot;data_logger&quot;)\n|&gt; level(TRACE)\n|&gt; append_instance(Direct(\n    None,\n    level_value(TRACE),\n    dispatcher(\n    builder()\n    |&gt; add_standard_log_message()\n    |&gt; add_data(data_serializer)\n    |&gt; build(),\n    ),\n))\ndata_logger\n|&gt; log_with_data(\nTRACE,\n&quot;Fetching user achievements&quot;,\nData(&quot;JohnBjrk&quot;, &quot;322f38e8-d0a5-43f2-9590-6f435a9b5e41&quot;),\n)\n```\n\n&gt; NOTE: `gleam_json` was imported as `import gleam/json as gjson` to avoid naming conflict with Glimts `json`module\n\nOf course if there is need for a more specific JSON serialization of the whole log-message a custom-written serializer can be supplied to the dispatcher.\n\n## Logger context\n\nSometimes it is useful for always log some data from the context. In order to do this we can use the `with_context` method which accepts a custom context and returns a logger which will include this context in all dispatched log-messages. The context can be serialized using the same strategy as for additional data using the `add_context` in the json serializer builder.\n\nThe following example adds some information from a `gleam_http` request to the log-message.\n\n```gleam\nlet request_serializer = fn(request: Request(BitString)) {\ngjson.object([\n    #(\n    &quot;request&quot;,\n    gjson.object([\n        #(&quot;method&quot;, gjson.string(method_to_string(request.method))),\n        #(&quot;path&quot;, gjson.string(request.path)),\n    ]),\n    ),\n])\n}\n\nlet context_logger =\nnew(&quot;context_logger&quot;)\n|&gt; level(TRACE)\n|&gt; append_instance(Direct(\n    None,\n    level_value(TRACE),\n    dispatcher(\n    builder()\n    |&gt; add_standard_log_message()\n    |&gt; add_context(request_serializer)\n    |&gt; build(),\n    ),\n))\n\nlet logger_with_request =\ncontext_logger\n|&gt; with_context(\n    request.new()\n    |&gt; set_path(&quot;api/login&quot;),\n)\n\nlogger_with_request\n|&gt; info(&quot;User successfully logged in&quot;)\n```\n\n## Erlang logger integration\n\nIt is possible to use the erlang built in [logger](https://www.erlang.org/doc/apps/kernel/logger_chapter.html) as a dispatcher in Glimt. There are two slightly different dispatchers that can be added to a logger to accomplish this: `logger_dispatch` and `logger_report_dispatch`. It is also possible to configure the erlang logger to use Glimt formatting. The following example uses the `basic_formatter` together with `logger_dispatch` to log an error. The `logger_dispatch` will add the message as a string.\n\n```gleam\nlet logger_logger =\nnew(&quot;logger_logger&quot;)\n|&gt; level(TRACE)\n|&gt; append_instance(Direct(None, level_value(TRACE), logger_dispatch))\n\nbasic_formatter.use_with_handler(&quot;default&quot;)\nlogger_logger\n|&gt; error(\n&quot;Could not connect to database&quot;,\nError(&quot;Connection timeout: 127:0.0.1:5432&quot;),\n)\n```\n\nThis will produce the following output:\n\n&gt; &lt;span style=&quot;color:gray&quot;&gt;2023-01-05T23:07:13.081161+01:00&lt;/span&gt; | &lt;span style=&quot;color:red&quot;&gt;error&lt;/span&gt; | &lt;span style=&quot;color:magenta&quot;&gt;logger_logger(&lt;0.9.0&gt;)&lt;/span&gt; | Could not connect to database | Error(&quot;Connection timeout: 127:0.0.1:5432&quot;)\n\nNote that we are setting the formatter for the `default` logger handler. This means that any log-message sent to the erlang logger will use this format.\n\nThe `logger_report_dispatch` will log a report instead of a plain string (which will include the message and other data). This is useful in combination with a logger with `context` and/or `data` since these two will be merged into the report. The following example combines the two:\n\n```gleam\nlet report_logger =\nnew(&quot;report_logger&quot;)\n|&gt; level(TRACE)\n|&gt; append_instance(Direct(None, level_value(TRACE), logger_report_dispatch))\n\nreport_logger\n|&gt; with_context([\n#(&quot;host&quot;, &quot;blue-panda.fly.dev&quot;),\n#(&quot;url&quot;, &quot;users&quot;),\n#(&quot;params&quot;, &quot;token=48dhf8h826ad876f78&amp;&quot;),\n])\n|&gt; log_with_data(\nINFO,\n&quot;Successfully fetched user data&quot;,\n[#(&quot;username&quot;, &quot;JohnBjrk&quot;), #(&quot;github_url&quot;, &quot;https://github.com/JohnBjrk&quot;)],\n)\n```\n\nIt gives the following output:\n\n&gt; &lt;span style=&quot;color:gray&quot;&gt;2023-01-05T23:07:13.084284+01:00&lt;/span&gt; | &lt;span style=&quot;color:green&quot;&gt;notice&lt;/span&gt; | &lt;span style=&quot;color:magenta&quot;&gt;report_logger(&lt;0.9.0&gt;)&lt;/span&gt; | (github_url =&gt; https://github.com/JohnBjrk, host =&gt; blue-panda.fly.dev, msg =&gt; Successfully fetched user data, params =&gt; token=48dhf8h826ad876f78&amp;, url =&gt; users, username =&gt; JohnBjrk)\n\nIt is also possible to make the output json from the erlang logger. The following line changes the `default` handler to output json:.\n\n```gleam\njson_formatter.use_with_handler(&quot;default&quot;)\n```\n\nWith this the two examples above will output the following:\n\n```json\n{&quot;error&quot;:&quot;Error(\\&quot;Connection timeout: 127:0.0.1:5432\\&quot;)&quot;,&quot;time&quot;:1672958146686409,&quot;time_string&quot;:&quot;2023-01-05T23:35:46.686409+01:00&quot;,&quot;message&quot;:&quot;Could not connect to database&quot;}\n{&quot;time&quot;:1672958146686453,&quot;time_string&quot;:&quot;2023-01-05T23:35:46.686453+01:00&quot;,&quot;github_url&quot;:&quot;https://github.com/JohnBjrk&quot;,&quot;host&quot;:&quot;blue-panda.fly.dev&quot;,&quot;msg&quot;:&quot;Successfully fetched user data&quot;,&quot;params&quot;:&quot;token=48dhf8h826ad876f78&amp;&quot;,&quot;url&quot;:&quot;users&quot;,&quot;username&quot;:&quot;JohnBjrk&quot;}\n```\n\n## Todo\n\nThese are some future improvements currently on the road-map for the Glimt library.\n\n- [ ] Add overload protection for Actor dispatchers\n- [ ] Add support for writing custom erlang logger handlers\n    - Utilize overload protection\n- [ ] Option to use basic formatters without color\n- [ ] File-based config for setting log-levels of loggers based on name\n    - Make it possible to override log-levels in libs by setting them in config file\n- [ ] File-dispatcher\n    - Write log messages to a file. Should probably be implemented as an actor.\n    - Support log-rotation?\n- [ ] External log service dispatchers\n    - Dispatch logs to an external service (rollbar, loggly ..). This is probably best done in a separate repository which implements a specific external service.\n- [ ] Dispatcher that use stderr\n    - Maybe there is a need to use stderr for output of error log-messages.\n","url":"index.html"},{"doc":"glimt/log_message","title":"LogLevel","content":"pub type LogLevel {\n  ALL\n  TRACE\n  DEBUG\n  INFO\n  WARNING\n  ERROR\n  FATAL\n  NONE\n}\n LogLevel used in log messages and as limits for loggers\nALL\n\n\nTRACE\n\n\nDEBUG\n\n\nINFO\n\n\nWARNING\n\n\nERROR\n\n\nFATAL\n\n\nNONE\n\n\nSynonyms:\nglimt/log_message.LogLevel\nglimt/log_message LogLevel","url":"glimt/log_message.html#LogLevel"},{"doc":"glimt/log_message","title":"LogMessage","content":"pub type LogMessage(data, context, result_type) {\n  LogMessage(\n    time: String,\n    name: String,\n    pid: Pid,\n    instance_name: Option(String),\n    instance_pid: Option(Pid),\n    level: LogLevel,\n    level_value: Int,\n    message: String,\n    error: Option(Result(result_type, result_type)),\n    data: Option(data),\n    context: Option(context),\n  )\n}\n LogMessage contains all the data that can be dispatched/serialized by a logger instance\nLogMessage(\n  time: String,\n  name: String,\n  pid: Pid,\n  instance_name: Option(String),\n  instance_pid: Option(Pid),\n  level: LogLevel,\n  level_value: Int,\n  message: String,\n  error: Option(Result(result_type, result_type)),\n  data: Option(data),\n  context: Option(context),\n)\n\n\nSynonyms:\nglimt/log_message.LogMessage\nglimt/log_message LogMessage","url":"glimt/log_message.html#LogMessage"},{"doc":"glimt/log_message","title":"level_string","content":"pub fn level_string(log_level: LogLevel) -&gt; String\n\nSynonyms:\nglimt/log_message.level_string\nglimt/log_message level_string","url":"glimt/log_message.html#level_string"},{"doc":"glimt/log_message","title":"level_value","content":"pub fn level_value(log_level: LogLevel) -&gt; Int\n\nSynonyms:\nglimt/log_message.level_value\nglimt/log_message level_value","url":"glimt/log_message.html#level_value"},{"doc":"glimt/log_message","title":"glimt/log_message","content":"","url":"glimt/log_message.html"},{"doc":"glimt/style","title":"style_debug","content":"pub fn style_debug() -&gt; fn(String) -&gt; String\n\nSynonyms:\nglimt/style.style_debug\nglimt/style style_debug","url":"glimt/style.html#style_debug"},{"doc":"glimt/style","title":"style_error","content":"pub fn style_error() -&gt; fn(String) -&gt; String\n\nSynonyms:\nglimt/style.style_error\nglimt/style style_error","url":"glimt/style.html#style_error"},{"doc":"glimt/style","title":"style_fatal","content":"pub fn style_fatal() -&gt; fn(String) -&gt; String\n\nSynonyms:\nglimt/style.style_fatal\nglimt/style style_fatal","url":"glimt/style.html#style_fatal"},{"doc":"glimt/style","title":"style_info","content":"pub fn style_info() -&gt; fn(String) -&gt; String\n\nSynonyms:\nglimt/style.style_info\nglimt/style style_info","url":"glimt/style.html#style_info"},{"doc":"glimt/style","title":"style_name","content":"pub fn style_name(name: String) -&gt; String\n\nSynonyms:\nglimt/style.style_name\nglimt/style style_name","url":"glimt/style.html#style_name"},{"doc":"glimt/style","title":"style_plain","content":"pub fn style_plain() -&gt; fn(String) -&gt; String\n\nSynonyms:\nglimt/style.style_plain\nglimt/style style_plain","url":"glimt/style.html#style_plain"},{"doc":"glimt/style","title":"style_time","content":"pub fn style_time(time: String) -&gt; String\n\nSynonyms:\nglimt/style.style_time\nglimt/style style_time","url":"glimt/style.html#style_time"},{"doc":"glimt/style","title":"style_trace","content":"pub fn style_trace() -&gt; fn(String) -&gt; String\n\nSynonyms:\nglimt/style.style_trace\nglimt/style style_trace","url":"glimt/style.html#style_trace"},{"doc":"glimt/style","title":"style_warning","content":"pub fn style_warning() -&gt; fn(String) -&gt; String\n\nSynonyms:\nglimt/style.style_warning\nglimt/style style_warning","url":"glimt/style.html#style_warning"},{"doc":"glimt/style","title":"glimt/style","content":"","url":"glimt/style.html"},{"doc":"glimt/serializer/basic","title":"basic_serializer","content":"pub fn basic_serializer(log_message: LogMessage(a, b, Dynamic)) -&gt; String\n Basic serializer that writes the message as one line separated by `|`\n &gt; NOTE: data and context will not be serialized\nSynonyms:\nglimt/serializer/basic.basic_serializer\nglimt/serializer/basic basic_serializer","url":"glimt/serializer/basic.html#basic_serializer"},{"doc":"glimt/serializer/basic","title":"level_symbol","content":"pub fn level_symbol(log_level: LogLevel) -&gt; String\n\nSynonyms:\nglimt/serializer/basic.level_symbol\nglimt/serializer/basic level_symbol","url":"glimt/serializer/basic.html#level_symbol"},{"doc":"glimt/serializer/basic","title":"glimt/serializer/basic","content":"","url":"glimt/serializer/basic.html"},{"doc":"glimt/serializer","title":"Serializer","content":"pub type Serializer(data, context, result_type) =\n  fn(LogMessage(data, context, result_type)) -&gt; String\n Represents a serializer of a LogMessage\n\nSynonyms:\nglimt/serializer.Serializer\nglimt/serializer Serializer","url":"glimt/serializer.html#Serializer"},{"doc":"glimt/serializer","title":"glimt/serializer","content":"","url":"glimt/serializer.html"},{"doc":"glimt/dispatcher/stdout","title":"dispatcher","content":"pub fn dispatcher(serializer: fn(LogMessage(a, b, c)) -&gt; String) -&gt; fn(\n  LogMessage(a, b, c),\n) -&gt; Nil\n Dispatcher that writes the log message to stdout\nSynonyms:\nglimt/dispatcher/stdout.dispatcher\nglimt/dispatcher/stdout dispatcher","url":"glimt/dispatcher/stdout.html#dispatcher"},{"doc":"glimt/dispatcher/stdout","title":"glimt/dispatcher/stdout","content":"","url":"glimt/dispatcher/stdout.html"},{"doc":"glimt","title":"Dispatcher","content":"pub type Dispatcher(data, context, result_type) =\n  fn(LogMessage(data, context, result_type)) -&gt; Nil\n Generic `Dispatcher` the can dispatch a `LogMessage` with a given `data` type and \n errors of a given type (build in dispatchers use `Dynamic` as error type)\n\nSynonyms:\nglimt.Dispatcher\nglimt Dispatcher","url":"glimt.html#Dispatcher"},{"doc":"glimt","title":"Logger","content":"pub opaque type Logger(data, context)\n Logger that can be use for logging of a `LogMessage` with possible additional `data`\n to one or more `LoggerInstance`\n\nSynonyms:\nglimt.Logger\nglimt Logger","url":"glimt.html#Logger"},{"doc":"glimt","title":"LoggerInstance","content":"pub type LoggerInstance(data, context) {\n  Direct(\n    name: Option(String),\n    level_min_value: Int,\n    dispatch: Dispatcher(data, context, Dynamic),\n  )\n  Actor(\n    name: Option(String),\n    level_min_value: Int,\n    Subject(LogMessage(data, context, Dynamic)),\n  )\n}\n A logger instance can be either `Direct` (logging in same process) or `Actor` (logging\n in a separate process) \nDirect(\n  name: Option(String),\n  level_min_value: Int,\n  dispatch: Dispatcher(data, context, Dynamic),\n)\n\n\nActor(\n  name: Option(String),\n  level_min_value: Int,\n  Subject(LogMessage(data, context, Dynamic)),\n)\n\n\nSynonyms:\nglimt.LoggerInstance\nglimt LoggerInstance","url":"glimt.html#LoggerInstance"},{"doc":"glimt","title":"append_instance","content":"pub fn append_instance(logger: Logger(a, b), instance: LoggerInstance(\n    a,\n    b,\n  )) -&gt; Logger(a, b)\n Add an instance to the logger\nSynonyms:\nglimt.append_instance\nglimt append_instance","url":"glimt.html#append_instance"},{"doc":"glimt","title":"debug","content":"pub fn debug(logger: Logger(a, b), message: String) -&gt; Logger(\n  a,\n  b,\n)\n Log a message at `DEBUG` level\nSynonyms:\nglimt.debug\nglimt debug","url":"glimt.html#debug"},{"doc":"glimt","title":"error","content":"pub fn error(logger: Logger(a, b), message: String, error: Result(\n    c,\n    d,\n  )) -&gt; Logger(a, b)\n Log a message at `ERROR` level with the provided `Result`\nSynonyms:\nglimt.error\nglimt error","url":"glimt.html#error"},{"doc":"glimt","title":"fatal","content":"pub fn fatal(logger: Logger(a, b), message: String, error: Result(\n    c,\n    d,\n  )) -&gt; Logger(a, b)\n Log a message at `FATAL` level with the provided `Result`\nSynonyms:\nglimt.fatal\nglimt fatal","url":"glimt.html#fatal"},{"doc":"glimt","title":"get_context","content":"pub fn get_context(logger: Logger(a, b)) -&gt; Option(b)\n Get current context data for the logger\nSynonyms:\nglimt.get_context\nglimt get_context","url":"glimt.html#get_context"},{"doc":"glimt","title":"info","content":"pub fn info(logger: Logger(a, b), message: String) -&gt; Logger(a, b)\n Log a message at `INFO` level\nSynonyms:\nglimt.info\nglimt info","url":"glimt.html#info"},{"doc":"glimt","title":"level","content":"pub fn level(logger: Logger(a, b), level: LogLevel) -&gt; Logger(\n  a,\n  b,\n)\n Set a new minimum level for the logger\nSynonyms:\nglimt.level\nglimt level","url":"glimt.html#level"},{"doc":"glimt","title":"log","content":"pub fn log(logger: Logger(a, b), level: LogLevel, message: String) -&gt; Logger(\n  a,\n  b,\n)\n Log a message at the provided level\nSynonyms:\nglimt.log\nglimt log","url":"glimt.html#log"},{"doc":"glimt","title":"log_error_with_data","content":"pub fn log_error_with_data(logger: Logger(a, b), level: LogLevel, message: String, error: Result(\n    c,\n    d,\n  ), data: a) -&gt; Logger(a, b)\n Log a message at the provided level together with the provided `Result` and data\nSynonyms:\nglimt.log_error_with_data\nglimt log_error_with_data","url":"glimt.html#log_error_with_data"},{"doc":"glimt","title":"log_with_data","content":"pub fn log_with_data(logger: Logger(a, b), level: LogLevel, message: String, data: a) -&gt; Logger(\n  a,\n  b,\n)\n Log a message at the provided level with some additional data\nSynonyms:\nglimt.log_with_data\nglimt log_with_data","url":"glimt.html#log_with_data"},{"doc":"glimt","title":"new","content":"pub fn new(name: String) -&gt; Logger(a, b)\n Create a new logger with the name &quot;anonymous&quot; that accepts any `LogLevel`\n The `Logger` starts without any [LoggerInstance](#LoggerInstance)\nSynonyms:\nglimt.new\nglimt new","url":"glimt.html#new"},{"doc":"glimt","title":"new_stdout","content":"pub fn new_stdout(name: String) -&gt; Logger(Nil, #(String, String))\n Convenience method for getting a direct stdout logger with basic serialization\n that accepts all log levels\nSynonyms:\nglimt.new_stdout\nglimt new_stdout","url":"glimt.html#new_stdout"},{"doc":"glimt","title":"start_instance","content":"pub fn start_instance(name: String, level: LogLevel, dispatch: fn(\n    LogMessage(a, b, c),\n  ) -&gt; Nil) -&gt; Result(LoggerInstance(a, b), StartError)\n Starts an actor instance that logs using the provided dispatcher and a\n specified level\nSynonyms:\nglimt.start_instance\nglimt start_instance","url":"glimt.html#start_instance"},{"doc":"glimt","title":"stdout_anonymous_instance","content":"pub fn stdout_anonymous_instance(level: LogLevel) -&gt; LoggerInstance(\n  a,\n  b,\n)\n Create an unnamed direct stdout instance with basic serialization\nSynonyms:\nglimt.stdout_anonymous_instance\nglimt stdout_anonymous_instance","url":"glimt.html#stdout_anonymous_instance"},{"doc":"glimt","title":"stdout_instance","content":"pub fn stdout_instance(name: String, level: LogLevel) -&gt; LoggerInstance(\n  a,\n  b,\n)\n Creates a direct stdout instance with basic serialization\nSynonyms:\nglimt.stdout_instance\nglimt stdout_instance","url":"glimt.html#stdout_instance"},{"doc":"glimt","title":"trace","content":"pub fn trace(logger: Logger(a, b), message: String) -&gt; Logger(\n  a,\n  b,\n)\n Log a message at `TRACE` level\nSynonyms:\nglimt.trace\nglimt trace","url":"glimt.html#trace"},{"doc":"glimt","title":"warning","content":"pub fn warning(logger: Logger(a, b), message: String) -&gt; Logger(\n  a,\n  b,\n)\n Log a message at `WARNING` level\nSynonyms:\nglimt.warning\nglimt warning","url":"glimt.html#warning"},{"doc":"glimt","title":"with_context","content":"pub fn with_context(logger: Logger(a, b), context: b) -&gt; Logger(\n  a,\n  b,\n)\n Add context data to the logger\nSynonyms:\nglimt.with_context\nglimt with_context","url":"glimt.html#with_context"},{"doc":"glimt","title":"with_time_provider","content":"pub fn with_time_provider(logger: Logger(a, b), time_provider: fn() -&gt;\n    String) -&gt; Logger(a, b)\n Set the function used to get time. The function should return a string representation\n of the current time. Can be used to configure the date/time format\nSynonyms:\nglimt.with_time_provider\nglimt with_time_provider","url":"glimt.html#with_time_provider"},{"doc":"glimt","title":"glimt","content":"","url":"glimt.html"},{"doc":"glimt/erlang_logger/level","title":"Level","content":"pub type Level {\n  Emergency\n  Alert\n  Critical\n  Error\n  Warning\n  Notice\n  Info\n  Debug\n}\n Erlang logger levels\nEmergency\n\n\nAlert\n\n\nCritical\n\n\nError\n\n\nWarning\n\n\nNotice\n\n\nInfo\n\n\nDebug\n\n\nSynonyms:\nglimt/erlang_logger/level.Level\nglimt/erlang_logger/level Level","url":"glimt/erlang_logger/level.html#Level"},{"doc":"glimt/erlang_logger/level","title":"glimt/erlang_logger/level","content":"","url":"glimt/erlang_logger/level.html"},{"doc":"glimt/erlang_logger/common","title":"a","content":"pub fn a(string: String) -&gt; Atom\n Shorthand for creating an `Atom`\n Equivalent to `atom.create_from_string()`\nSynonyms:\nglimt/erlang_logger/common.a\nglimt/erlang_logger/common a","url":"glimt/erlang_logger/common.html#a"},{"doc":"glimt/erlang_logger/common","title":"built_in_format","content":"pub fn built_in_format(log_event: Dynamic, config: Dynamic) -&gt; String\n Calls the standard erlang formatter. Useful when a formatter is unable to\n format the log_event\nSynonyms:\nglimt/erlang_logger/common.built_in_format\nglimt/erlang_logger/common built_in_format","url":"glimt/erlang_logger/common.html#built_in_format"},{"doc":"glimt/erlang_logger/common","title":"format_dynamic","content":"pub fn format_dynamic(dynamic_value: Dynamic) -&gt; String\n String representation of a `Dynamic` value.\n If the dynamic value is a String it will be returned, otherwise the value\n will be converted using `erlang.format`\nSynonyms:\nglimt/erlang_logger/common.format_dynamic\nglimt/erlang_logger/common format_dynamic","url":"glimt/erlang_logger/common.html#format_dynamic"},{"doc":"glimt/erlang_logger/common","title":"logger_format","content":"pub external fn logger_format(\n  log_event: Dynamic,\n  config: Dynamic,\n) -&gt; Charlist\n\nSynonyms:\nglimt/erlang_logger/common.logger_format\nglimt/erlang_logger/common logger_format","url":"glimt/erlang_logger/common.html#logger_format"},{"doc":"glimt/erlang_logger/common","title":"set_handler_config","content":"pub external fn set_handler_config(\n  handler_id: Atom,\n  config_item: Atom,\n  config: #(Atom, Dynamic),\n) -&gt; Nil\n Set config for handler with `handler_id`\nSynonyms:\nglimt/erlang_logger/common.set_handler_config\nglimt/erlang_logger/common set_handler_config","url":"glimt/erlang_logger/common.html#set_handler_config"},{"doc":"glimt/erlang_logger/common","title":"time_to_string","content":"pub fn time_to_string(time: Int) -&gt; String\n Converts a epoch microsecond value to a rfc3339 string\nSynonyms:\nglimt/erlang_logger/common.time_to_string\nglimt/erlang_logger/common time_to_string","url":"glimt/erlang_logger/common.html#time_to_string"},{"doc":"glimt/erlang_logger/common","title":"glimt/erlang_logger/common","content":" Common functions used by the `logger`, `basic_formatter` and `json_formatter`","url":"glimt/erlang_logger/common.html"},{"doc":"glimt/erlang_logger/logger","title":"logger_dispatch","content":"pub fn logger_dispatch(log_message: LogMessage(a, b, Dynamic)) -&gt; Nil\n Dispatcher that send the `LogMessage` to the erlang built-in logger\n Logs the message as a string\nSynonyms:\nglimt/erlang_logger/logger.logger_dispatch\nglimt/erlang_logger/logger logger_dispatch","url":"glimt/erlang_logger/logger.html#logger_dispatch"},{"doc":"glimt/erlang_logger/logger","title":"logger_report_dispatch","content":"pub fn logger_report_dispatch(log_message: LogMessage(\n    List(#(String, String)),\n    List(#(String, String)),\n    Dynamic,\n  )) -&gt; Nil\n Dispatcher that send the `LogMessage` to the erlang built-in logger\n Logs the message as `msg` field in a report\n Merges `data` and `context` into the report\nSynonyms:\nglimt/erlang_logger/logger.logger_report_dispatch\nglimt/erlang_logger/logger logger_report_dispatch","url":"glimt/erlang_logger/logger.html#logger_report_dispatch"},{"doc":"glimt/erlang_logger/logger","title":"glimt/erlang_logger/logger","content":"","url":"glimt/erlang_logger/logger.html"},{"doc":"glimt/serializer/json","title":"JsonSerializerBuilder","content":"pub type JsonSerializerBuilder(data, context, result_type) =\n  fn(LogMessage(data, context, result_type)) -&gt;\n    List(#(String, Json))\n\n\nSynonyms:\nglimt/serializer/json.JsonSerializerBuilder\nglimt/serializer/json JsonSerializerBuilder","url":"glimt/serializer/json.html#JsonSerializerBuilder"},{"doc":"glimt/serializer/json","title":"add_context","content":"pub fn add_context(builder: fn(LogMessage(a, b, c)) -&gt;\n    List(#(String, Json)), context_serializer: fn(b) -&gt; Json) -&gt; fn(\n  LogMessage(a, b, c),\n) -&gt; List(#(String, Json))\n Add serializer for context data\nSynonyms:\nglimt/serializer/json.add_context\nglimt/serializer/json add_context","url":"glimt/serializer/json.html#add_context"},{"doc":"glimt/serializer/json","title":"add_data","content":"pub fn add_data(builder: fn(LogMessage(a, b, c)) -&gt;\n    List(#(String, Json)), data_serializer: fn(a) -&gt; Json) -&gt; fn(\n  LogMessage(a, b, c),\n) -&gt; List(#(String, Json))\n Add serializer for additional data\nSynonyms:\nglimt/serializer/json.add_data\nglimt/serializer/json add_data","url":"glimt/serializer/json.html#add_data"},{"doc":"glimt/serializer/json","title":"add_standard_log_message","content":"pub fn add_standard_log_message(builder: fn(LogMessage(a, b, c)) -&gt;\n    List(#(String, Json))) -&gt; fn(LogMessage(a, b, c)) -&gt;\n  List(#(String, Json))\n Add standard log message to serializer\nSynonyms:\nglimt/serializer/json.add_standard_log_message\nglimt/serializer/json add_standard_log_message","url":"glimt/serializer/json.html#add_standard_log_message"},{"doc":"glimt/serializer/json","title":"build","content":"pub fn build(builder: fn(LogMessage(a, b, c)) -&gt;\n    List(#(String, Json))) -&gt; fn(LogMessage(a, b, c)) -&gt; String\n Create a serializer from builder\nSynonyms:\nglimt/serializer/json.build\nglimt/serializer/json build","url":"glimt/serializer/json.html#build"},{"doc":"glimt/serializer/json","title":"builder","content":"pub fn builder() -&gt; fn(LogMessage(a, b, c)) -&gt;\n  List(#(String, Json))\n Create a JSON serializer builder\nSynonyms:\nglimt/serializer/json.builder\nglimt/serializer/json builder","url":"glimt/serializer/json.html#builder"},{"doc":"glimt/serializer/json","title":"new_json_serializer","content":"pub fn new_json_serializer() -&gt; fn(LogMessage(a, b, c)) -&gt; String\n Creates a new JSON serializer that includes standard log data\nSynonyms:\nglimt/serializer/json.new_json_serializer\nglimt/serializer/json new_json_serializer","url":"glimt/serializer/json.html#new_json_serializer"},{"doc":"glimt/serializer/json","title":"glimt/serializer/json","content":"","url":"glimt/serializer/json.html"},{"doc":"glimt/erlang_logger/log_event","title":"LogEvent","content":"pub type LogEvent {\n  Message(\n    time_us: Int,\n    level: Level,\n    message: String,\n    pid: Pid,\n    logger_name: Option(String),\n    error: Option(String),\n  )\n  Report(\n    time_us: Int,\n    level: Level,\n    report: List(#(Dynamic, Dynamic)),\n    pid: Pid,\n    logger_name: Option(String),\n    error: Option(String),\n  )\n}\n Gleam representation of an erlang log event\nMessage(\n  time_us: Int,\n  level: Level,\n  message: String,\n  pid: Pid,\n  logger_name: Option(String),\n  error: Option(String),\n)\n\n\nReport(\n  time_us: Int,\n  level: Level,\n  report: List(#(Dynamic, Dynamic)),\n  pid: Pid,\n  logger_name: Option(String),\n  error: Option(String),\n)\n\n\nSynonyms:\nglimt/erlang_logger/log_event.LogEvent\nglimt/erlang_logger/log_event LogEvent","url":"glimt/erlang_logger/log_event.html#LogEvent"},{"doc":"glimt/erlang_logger/log_event","title":"decode_log_event","content":"pub fn decode_log_event(log_event: Dynamic) -&gt; Result(\n  LogEvent,\n  List(DecodeError),\n)\n Translate a log event from erlang logger to corresponding gleam representation\n This function is useful when implementing custom formatters for the erlang logger\nSynonyms:\nglimt/erlang_logger/log_event.decode_log_event\nglimt/erlang_logger/log_event decode_log_event","url":"glimt/erlang_logger/log_event.html#decode_log_event"},{"doc":"glimt/erlang_logger/log_event","title":"glimt/erlang_logger/log_event","content":"","url":"glimt/erlang_logger/log_event.html"},{"doc":"glimt/erlang_logger/basic_formatter","title":"format","content":"pub fn format(log_event: Dynamic, config: Dynamic) -&gt; String\n This is the callback that will be used by erlang logger to format\n log events. It is not intended to use directly.\nSynonyms:\nglimt/erlang_logger/basic_formatter.format\nglimt/erlang_logger/basic_formatter format","url":"glimt/erlang_logger/basic_formatter.html#format"},{"doc":"glimt/erlang_logger/basic_formatter","title":"use_with_handler","content":"pub fn use_with_handler(handler_id: String) -&gt; Nil\n Set `basic_formatter` for erlang logger [handler](https://www.erlang.org/doc/apps/kernel/logger_chapter.html#handlers)\n with `handler_id`\n\n Example:\n \n use_with_handler(&quot;default&quot;)\n \n\n Set basic Glimt formatting for the default handler (logger_std_h)\nSynonyms:\nglimt/erlang_logger/basic_formatter.use_with_handler\nglimt/erlang_logger/basic_formatter use_with_handler","url":"glimt/erlang_logger/basic_formatter.html#use_with_handler"},{"doc":"glimt/erlang_logger/basic_formatter","title":"glimt/erlang_logger/basic_formatter","content":"","url":"glimt/erlang_logger/basic_formatter.html"},{"doc":"glimt/erlang_logger/json_formatter","title":"format","content":"pub fn format(log_event: Dynamic, config: Dynamic) -&gt; String\n This is the callback that will be used by erlang logger to format\n log events. It is not intended to use directly.\nSynonyms:\nglimt/erlang_logger/json_formatter.format\nglimt/erlang_logger/json_formatter format","url":"glimt/erlang_logger/json_formatter.html#format"},{"doc":"glimt/erlang_logger/json_formatter","title":"use_with_handler","content":"pub fn use_with_handler(handler_id: String) -&gt; Nil\n Set `json_formatter` for erlang logger [handler](https://www.erlang.org/doc/apps/kernel/logger_chapter.html#handlers)\n with `handler_id`\n\n Example:\n \n use_with_handler(&quot;default&quot;)\n \n\n Set basic Glimt formatting for the default handler (logger_std_h)\nSynonyms:\nglimt/erlang_logger/json_formatter.use_with_handler\nglimt/erlang_logger/json_formatter use_with_handler","url":"glimt/erlang_logger/json_formatter.html#use_with_handler"},{"doc":"glimt/erlang_logger/json_formatter","title":"glimt/erlang_logger/json_formatter","content":"","url":"glimt/erlang_logger/json_formatter.html"}]);